"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConnectionManager = void 0;
const Connection_1 = require("./Connection");
const ConnectionError_1 = require("./ConnectionError");
const promise_1 = require("mysql2/promise");
const Session_1 = require("./Session");
class ConnectionManager {
    /** Create a new connection manager with given options */
    constructor(options) {
        this.closed = false;
        this.pool = (0, promise_1.createPool)({
            host: options.host,
            port: options.port,
            user: options.user,
            password: options.password,
            database: options.database,
            charset: options.charset || 'utf8mb4',
            connectionLimit: options.connectionLimit,
            queueLimit: options.queueLimit || 0,
            waitForConnections: (options.queueLimit || 0) > 0,
            multipleStatements: options.multipleStatements,
            insecureAuth: options.insecureAuth || false,
            rowsAsArray: true,
            namedPlaceholders: true,
            supportBigNumbers: true,
            bigNumberStrings: true,
            dateStrings: true,
            timezone: 'Z'
        });
        this.logger = options.logger || {};
        this.releaseStrategy = options.releaseStrategy || 'RESET_STATE';
    }
    /** Run the handler within an automatically managed connection */
    async connection(handler, options) {
        if (this.closed) {
            throw new ConnectionError_1.ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
        }
        const connection = await this.getConnection(options);
        try {
            return await handler(connection);
        }
        finally {
            await connection.release();
        }
    }
    /** Run the handler within an automatically managed session */
    async session(handler, options) {
        if (this.closed) {
            throw new ConnectionError_1.ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
        }
        const session = await this.getSession(options);
        try {
            return await handler(session);
        }
        finally {
            session.close();
        }
    }
    /** Get a manually managed connection (do not forget to release it when done) */
    async getConnection(options) {
        if (this.closed) {
            throw new ConnectionError_1.ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
        }
        try {
            const poolConnection = await this.pool.getConnection();
            const logger = options?.logger || this.logger;
            const releaseStrategy = options?.releaseStrategy || this.releaseStrategy;
            if (logger.acquire) {
                logger.acquire(poolConnection.threadId);
            }
            return new Connection_1.Connection(poolConnection, logger, releaseStrategy);
        }
        catch (err) {
            throw ConnectionError_1.ConnectionError.fromError(err);
        }
    }
    /** Get a manually managed session (unlike connection, closing a session is completely optional) */
    getSession(options) {
        if (this.closed) {
            throw new ConnectionError_1.ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
        }
        const settings = {
            logger: options?.logger || this.logger,
            releaseStrategy: options?.releaseStrategy || this.releaseStrategy
        };
        return new Session_1.Session(this, settings);
    }
    /** Close the connection manager, gracefully close all connections and stop waiting for new ones */
    async close() {
        if (this.closed) {
            return;
        }
        this.closed = true;
        try {
            await this.pool.end();
        }
        catch (err) {
            throw ConnectionError_1.ConnectionError.fromError(err);
        }
    }
    /** Is the connection manager closed */
    get isClosed() {
        return this.closed;
    }
}
exports.ConnectionManager = ConnectionManager;
