"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManagedConnection = void 0;
const QueryExecutor_1 = require("./QueryExecutor");
const Transaction_1 = require("./Transaction");
const TransactionError_1 = require("./TransactionError");
class ManagedConnection extends QueryExecutor_1.QueryExecutor {
    /** Run "START TRANSACTION" query if not already in transaction (throws a QueryError otherwise) */
    async startTransaction(mode) {
        if (this.inTx) {
            throw new TransactionError_1.TransactionError(`Unable to start a transaction while another transaction is in progress`);
        }
        await this.query(`START TRANSACTION${mode === 'READ' ? ' READ ONLY' : mode === 'WRITE' ? ' READ WRITE' : ''}`);
    }
    /** Run "COMMIT" query if in transaction (throws a QueryError otherwise) */
    async commit() {
        if (!this.inTx) {
            throw new TransactionError_1.TransactionError(`Unable to commit when transaction has not been started`);
        }
        await this.query(`COMMIT`);
    }
    /** Run "ROLLBACK" query if in transaction (throws a QueryError otherwise) */
    async rollback() {
        if (!this.inTx) {
            throw new TransactionError_1.TransactionError(`Unable to rollback when transaction has not been started`);
        }
        await this.query(`ROLLBACK`);
    }
    /** Does the current connection have an active transaction */
    get hasActiveTransaction() {
        return this.inTx;
    }
    /** Run the handler in an automatically managed transaction */
    async transaction(handler, options) {
        if (this.inTx) {
            throw new TransactionError_1.TransactionError(`Unable to start a transaction while another transaction is in progress`);
        }
        if (this.busy) {
            throw new TransactionError_1.TransactionError(`Unable to start a transaction while another operation is in progress`);
        }
        let result;
        this.busy = true;
        this.inTx = true;
        try {
            const logger = options?.logger || this.logger;
            const tx = await Transaction_1.Transaction.start(this.poolConnection, logger, options?.mode);
            try {
                result = await handler(tx);
            }
            catch (err) {
                try {
                    await Transaction_1.Transaction.rollback(tx);
                }
                catch (rbErr) {
                    // discard rollback error if any in favor of the original error
                }
                throw err;
            }
            await Transaction_1.Transaction.commit(tx);
        }
        finally {
            this.busy = false;
            this.inTx = false;
        }
        return result;
    }
}
exports.ManagedConnection = ManagedConnection;
