"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManagedSession = void 0;
const Formatter_1 = require("./Formatter");
const SessionError_1 = require("./SessionError");
class ManagedSession {
    /** INTERNAL, DO NOT USE DIRECTLY */
    constructor(connectionManager, settings) {
        this.connectionManager = connectionManager;
        this.settings = settings;
        this.closed = false;
    }
    /**
     * Escape a value
     * @see https://github.com/mysqljs/mysql#escaping-query-values
     */
    escape(value) {
        return Formatter_1.Formatter.escape(value);
    }
    /**
     * Escape an identifier (e.g. table or column name) with backticks
     */
    escapeId(id) {
        return Formatter_1.Formatter.escapeId(id);
    }
    /** Execute a query with given parameters in an automatically managed connection temporarily acquired from pool */
    async query(sql, parameters) {
        if (this.closed) {
            throw new SessionError_1.SessionError(`Session has been closed and is no longer available to perform operations`);
        }
        const connection = await this.connectionManager.getConnection(this.settings);
        try {
            return await connection.query(sql, parameters);
        }
        finally {
            await connection.release();
        }
    }
    /** Execute the handler in an automatically managed transaction within a temporary connection acquired from pool */
    async transaction(handler, options) {
        if (this.closed) {
            throw new SessionError_1.SessionError(`Session has been closed and is no longer available to perform operations`);
        }
        const connection = await this.connectionManager.getConnection(this.settings);
        try {
            return await connection.transaction(handler, options);
        }
        finally {
            await connection.release();
        }
    }
}
exports.ManagedSession = ManagedSession;
