"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryExecutor = void 0;
const Formatter_1 = require("./Formatter");
const QueryError_1 = require("./QueryError");
const QueryResult_1 = require("./QueryResult");
class QueryExecutor {
    constructor(poolConnection, logger) {
        this.poolConnection = poolConnection;
        this.logger = logger;
        this.inTx = false;
        this.busy = false;
        this.closed = false;
    }
    /**
     * Escape a value
     * @see https://github.com/mysqljs/mysql#escaping-query-values
     */
    escape(value) {
        return Formatter_1.Formatter.escape(value);
    }
    /**
     * Escape an identifier (e.g. table or column name) with backticks
     */
    escapeId(id) {
        return Formatter_1.Formatter.escapeId(id);
    }
    /** Execute a query with given parameters */
    async query(sql, parameters) {
        if (this.logger.query) {
            this.logger.query(sql, parameters, this.poolConnection.threadId);
        }
        if (this.closed) {
            throw new QueryError_1.QueryError(`Query executor has been closed and is no longer available to perform operations`);
        }
        if (this.busy) {
            throw new QueryError_1.QueryError(`Query executor is busy with another operation`);
        }
        this.busy = true;
        try {
            const [result, fields] = await this.poolConnection.query({ sql, values: parameters });
            const isSingleRs = typeof fields === 'undefined' ||
                Array.isArray(fields) && typeof fields[0] !== 'undefined' && !Array.isArray(fields[0]);
            const rsList = isSingleRs ? [result] : result;
            const flList = isSingleRs ? [fields] : fields;
            for (const rs of rsList) {
                if (typeof rs === 'object' && typeof rs.serverStatus === 'number') {
                    this.inTx = rs.serverStatus % 2 === 1;
                }
            }
            if (rsList.length !== flList.length) {
                throw new Error(`Result and fields arrays length mismatch`);
            }
            const len = rsList.length;
            let qr = new QueryResult_1.QueryResult(flList[len - 1], rsList[len - 1], null);
            for (let i = len - 2; i >= 0; i--) {
                qr = new QueryResult_1.QueryResult(flList[i], rsList[i], qr);
            }
            if (this.logger.result) {
                this.logger.result(qr, this.poolConnection.threadId);
            }
            return qr;
        }
        catch (err) {
            throw QueryError_1.QueryError.fromError(err);
        }
        finally {
            this.busy = false;
        }
    }
    /** Is the current query executor busy with executing a query */
    get isBusy() {
        return this.busy;
    }
}
exports.QueryExecutor = QueryExecutor;
