
import { PoolConnection } from 'mysql2/promise';
import { ConnectionError } from './ConnectionError';
import { type ConnectionReleaseStrategy } from './ConnectionReleaseStrategy';
import { ManagedConnection } from './ManagedConnection';
import { Logger } from './Logger';

export class Connection extends ManagedConnection {

	/** INTERNAL, DO NOT USE DIRECTLY */
	public constructor(
		poolConnection: PoolConnection,
		logger: Logger,
		private releaseStrategy: ConnectionReleaseStrategy
	) {
		super(poolConnection, logger);
	}

	/**
	 * Release current connection back to the pool according to selected connection release strategy
	 */
	public async release() {
		if (this.closed) {
			return;
		}
		if (this.busy) {
			throw new ConnectionError(`Unable to release a connection while an operation is in progress`);
		}
		this.closed = true;
		const promise = this.releaseToPool();
		if (this.logger.release) {
			this.logger.release(this.poolConnection.threadId);
		}
		if (this.releaseStrategy === 'WAIT_FOR_STATE_RESET') {
			await promise;
		}
	}

	/** INTERNAL, DO NOT USE DIRECTLY */
	private async releaseToPool() {
		if (this.releaseStrategy !== 'KEEP_STATE') {
			try {
				await this.poolConnection.changeUser({ });
			} catch (err) {
				// ignore change user errors as they'll invalidate the connection when reaching pool
			}
		}
		this.poolConnection.release();
	}

	/** Is the connection released */
	public get isReleased() {
		return this.closed;
	}

}
