
import { Connection } from './Connection';
import { ConnectionError } from './ConnectionError';
import { type ConnectionManagerOptions } from "./ConnectionManagerOptions";
import { createPool, Pool } from 'mysql2/promise';
import { ManagedConnection } from './ManagedConnection';
import { Session } from './Session';
import { ManagedSession } from './ManagedSession';
import { Logger } from './Logger';
import { ConnectionOptions } from './ConnectionOptions';
import { type ConnectionReleaseStrategy } from './ConnectionReleaseStrategy';

export class ConnectionManager {

	private pool: Pool;
	private closed: boolean = false;
	private logger: Logger;
	private releaseStrategy: ConnectionReleaseStrategy;

	/** Create a new connection manager with given options */
	public constructor(options: ConnectionManagerOptions) {
		this.pool = createPool({
			host: options.host,
			port: options.port,
			user: options.user,
			password: options.password,
			database: options.database,
			charset: options.charset || 'utf8mb4',
			connectionLimit: options.connectionLimit,
			queueLimit: options.queueLimit || 0,
			waitForConnections: (options.queueLimit || 0) > 0,
			multipleStatements: options.multipleStatements,
			insecureAuth: options.insecureAuth || false,
			rowsAsArray: true,
			namedPlaceholders: true,
			supportBigNumbers: true,
			bigNumberStrings: true,
			dateStrings: true,
			timezone: 'Z'
		});
		this.logger = options.logger || { };
		this.releaseStrategy = options.releaseStrategy || 'RESET_STATE';
	}

	/** Run the handler within an automatically managed connection */
	public async connection<T>(handler: (connection: ManagedConnection) => Promise<T>, options?: ConnectionOptions) {
		if (this.closed) {
			throw new ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
		}
		const connection = await this.getConnection(options);
		try {
			return await handler(connection);
		} finally {
			await connection.release();
		}
	}

	/** Run the handler within an automatically managed session */
	public async session<T>(handler: (session: ManagedSession) => Promise<T>, options?: ConnectionOptions) {
		if (this.closed) {
			throw new ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
		}
		const session = await this.getSession(options);
		try {
			return await handler(session);
		} finally {
			session.close();
		}
	}

	/** Get a manually managed connection (do not forget to release it when done) */
	public async getConnection(options?: ConnectionOptions) {
		if (this.closed) {
			throw new ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
		}
		try {
			const poolConnection = await this.pool.getConnection();
			const logger = options?.logger || this.logger;
			const releaseStrategy = options?.releaseStrategy || this.releaseStrategy;
			if (logger.acquire) {
				logger.acquire(poolConnection.threadId);
			}
			return new Connection(poolConnection, logger, releaseStrategy);
		} catch (err) {
			throw ConnectionError.fromError(err);
		}
	}

	/** Get a manually managed session (unlike connection, closing a session is completely optional) */
	public getSession(options?: ConnectionOptions) {
		if (this.closed) {
			throw new ConnectionError(`Connection manager has been closed and is no longer available to perform operations`);
		}
		const settings = {
			logger: options?.logger || this.logger,
			releaseStrategy: options?.releaseStrategy || this.releaseStrategy
		};
		return new Session(this, settings);
	}

	/** Close the connection manager, gracefully close all connections and stop waiting for new ones */
	public async close() {
		if (this.closed) {
			return;
		}
		this.closed = true;
		try {
			await this.pool.end();
		} catch(err) {
			throw ConnectionError.fromError(err);
		}
	}

	/** Is the connection manager closed */
	public get isClosed() {
		return this.closed;
	}

}
