
import { QueryExecutor } from './QueryExecutor';
import { Transaction } from './Transaction';
import { TransactionError } from './TransactionError';
import { TransactionOptions } from './TransactionOptions';

export class ManagedConnection extends QueryExecutor {

	/** Run "START TRANSACTION" query if not already in transaction (throws a QueryError otherwise) */
	public async startTransaction(mode?: 'READ' | 'WRITE') {
		if (this.inTx) {
			throw new TransactionError(`Unable to start a transaction while another transaction is in progress`);
		}
		await this.query(`START TRANSACTION${mode === 'READ' ? ' READ ONLY' : mode === 'WRITE' ? ' READ WRITE' : ''}`);
	}

	/** Run "COMMIT" query if in transaction (throws a QueryError otherwise) */
	public async commit() {
		if (!this.inTx) {
			throw new TransactionError(`Unable to commit when transaction has not been started`);
		}
		await this.query(`COMMIT`);
	}

	/** Run "ROLLBACK" query if in transaction (throws a QueryError otherwise) */
	public async rollback() {
		if (!this.inTx) {
			throw new TransactionError(`Unable to rollback when transaction has not been started`);
		}
		await this.query(`ROLLBACK`);
	}

	/** Does the current connection have an active transaction */
	public get hasActiveTransaction() {
		return this.inTx;
	}

	/** Run the handler in an automatically managed transaction */
	public async transaction<T>(handler: (tx: Transaction) => Promise<T>, options?: TransactionOptions) {
		if (this.inTx) {
			throw new TransactionError(`Unable to start a transaction while another transaction is in progress`);
		}
		if (this.busy) {
			throw new TransactionError(`Unable to start a transaction while another operation is in progress`);
		}
		let result;
		this.busy = true;
		this.inTx = true;
		try {
			const logger = options?.logger || this.logger;
			const tx = await Transaction.start(this.poolConnection, logger, options?.mode);
			try {
				result = await handler(tx);
			} catch(err) {
				try {
					await Transaction.rollback(tx);
				} catch (rbErr) {
					// discard rollback error if any in favor of the original error
				}
				throw err;
			}
			await Transaction.commit(tx);
		} finally {
			this.busy = false;
			this.inTx = false;
		}
		return result;
	}

}
