import { ConnectionManager } from "./ConnectionManager";
import { ConnectionOptions } from "./ConnectionOptions";
import { Formatter } from "./Formatter";
import { Escapable, Queryable, QueryParameters } from "./Queryable";
import { SessionError } from "./SessionError";
import { Transaction } from "./Transaction";
import { TransactionOptions } from "./TransactionOptions";

export class ManagedSession implements Queryable {

	protected closed: boolean = false;

	/** INTERNAL, DO NOT USE DIRECTLY */
	public constructor(
		private connectionManager: ConnectionManager,
		private settings: Required<ConnectionOptions>
	) { }

	/**
	 * Escape a value
	 * @see https://github.com/mysqljs/mysql#escaping-query-values
	 */
	public escape(value: Escapable) {
		return Formatter.escape(value);
	}

	/**
	 * Escape an identifier (e.g. table or column name) with backticks
	 */
	public escapeId(id: string) {
		return Formatter.escapeId(id);
	}

	/** Execute a query with given parameters in an automatically managed connection temporarily acquired from pool */
	public async query(sql: string, parameters?: QueryParameters) {
		if (this.closed) {
			throw new SessionError(`Session has been closed and is no longer available to perform operations`);
		}
		const connection = await this.connectionManager.getConnection(this.settings);
		try {
			return await connection.query(sql, parameters);
		} finally {
			await connection.release();
		}
	}

	/** Execute the handler in an automatically managed transaction within a temporary connection acquired from pool */
	public async transaction<T>(handler: (tx: Transaction) => Promise<T>, options?: TransactionOptions) {
		if (this.closed) {
			throw new SessionError(`Session has been closed and is no longer available to perform operations`);
		}
		const connection = await this.connectionManager.getConnection(this.settings);
		try {
			return await connection.transaction(handler, options);
		} finally {
			await connection.release();
		}
	}

}
