
import { PoolConnection } from 'mysql2/promise';
import { Formatter } from './Formatter';
import { Logger } from './Logger';
import { Escapable, Queryable, QueryParameters } from './Queryable';
import { QueryError } from './QueryError';
import { QueryResult } from './QueryResult';

export class QueryExecutor implements Queryable {

	protected inTx: boolean = false;
	protected busy: boolean = false;
	protected closed: boolean = false;

	protected constructor(
		protected poolConnection: PoolConnection,
		protected logger: Logger
	) { }

	/**
	 * Escape a value
	 * @see https://github.com/mysqljs/mysql#escaping-query-values
	 */
	public escape(value: Escapable) {
		return Formatter.escape(value);
	}

	/**
	 * Escape an identifier (e.g. table or column name) with backticks
	 */
	public escapeId(id: string) {
		return Formatter.escapeId(id);
	}

	/** Execute a query with given parameters */
	public async query(sql: string, parameters?: QueryParameters) {
		if (this.logger.query) {
			this.logger.query(sql, parameters, this.poolConnection.threadId);
		}
		if (this.closed) {
			throw new QueryError(`Query executor has been closed and is no longer available to perform operations`);
		}
		if (this.busy) {
			throw new QueryError(`Query executor is busy with another operation`);
		}
		this.busy = true;
		try {
			const [ result, fields ] = await this.poolConnection.query({ sql, values: parameters });
			const isSingleRs = typeof fields === 'undefined' ||
				Array.isArray(fields) && typeof fields[0] !== 'undefined' && !Array.isArray(fields[0]);
			const rsList: any[] = isSingleRs ? [ result ] : result as any[];
			const flList: any[] = isSingleRs ? [ fields ] : fields as any[];
			for (const rs of rsList) {
				if (typeof rs === 'object' && typeof rs.serverStatus === 'number') {
					this.inTx = rs.serverStatus % 2 === 1;
				}
			}
			if (rsList.length !== flList.length) {
				throw new Error(`Result and fields arrays length mismatch`);
			}
			const len = rsList.length;
			let qr = new QueryResult(flList[len - 1], rsList[len - 1], null);
			for (let i = len - 2; i >= 0; i--) {
				qr = new QueryResult(flList[i], rsList[i], qr);
			}
			if (this.logger.result) {
				this.logger.result(qr, this.poolConnection.threadId);
			}
			return qr;
		} catch (err) {
			throw QueryError.fromError(err);
		} finally {
			this.busy = false;
		}
	}

	/** Is the current query executor busy with executing a query */
	public get isBusy() {
		return this.busy;
	}

}
