
import { QueryError } from "./QueryError";
import { QueryResultColumn } from "./QueryResultColumn";

export class QueryResult {

	/** A list of columns of the result set (empty array if not a query result) */
	readonly columns: readonly QueryResultColumn[];

	/** A list of rows of the result set (empty array if not a query result) */
	readonly rows: readonly unknown[][];

	/** Last inserted identifier (i.e. LAST_INSERT_ID()) for an INSERT statement */
	readonly lastInsertId: string;

	/** A number of affected row for an UPDATE or DELETE statement */
	readonly affectedRows: number;

	/** A number of warnings produced by this query execution */
	readonly warningsCount: number;

	/** INTERNAL, DO NOT USE DIRECTLY */
	public constructor(fields: any, data: any, private next: QueryResult | null) {
		this.columns = Array.isArray(fields)
			? fields.map(field => convertColumnDefinition(field))
			: [ ];
		if (Array.isArray(data)) {
			this.rows = data;
			this.lastInsertId = '0';
			this.affectedRows = 0;
			this.warningsCount = 0;
		} else {
			this.rows = [ ];
			this.lastInsertId = String(data.insertId);
			this.affectedRows = parseFloat(data.affectedRows);
			this.warningsCount = parseFloat(data.warningStatus);
		}
	}

	public get isResultSet() {
		return this.columns.length > 0;
	}

	/** Translate the result into an array of objects, optionally prefixing each field with table and/or schema name */
	public asObjects(tableNameSeparator?: string, schemaNameSeparator?: string) {
		return this.rows.map(
			(fields) => {
				const obj: { [k: string]: unknown } = { };
				for (let i = 0; i < fields.length; i++) {
					const schemaPrefix = typeof schemaNameSeparator === 'string'
						? `${this.columns[i].schema}${schemaNameSeparator}`
						: ``;
					const tablePrefix = typeof tableNameSeparator === 'string'
						? `${this.columns[i].table}${tableNameSeparator}`
						: ``;
					obj[`${schemaPrefix}${tablePrefix}${this.columns[i].name}`] = fields[i];
				}
				return obj;
			}
		);
	}

	/** Get the next result of a multi-statement query */
	public async getNextResult() {
		if (this.next === null) {
			throw new QueryError(`There is no next result available`);
		}
		return this.next;
	}

	/** Whether there may be a next result in a multi-statement query */
	public get hasNextResult() {
		return this.next !== null;
	}

}

function convertColumnDefinition(def: any): QueryResultColumn {
	return {
		schema: def._buf.toString(def._clientEncoding, def._schemaStart, def._schemaStart + def._schemaLength),
		table: def._buf.toString(def._clientEncoding, def._tableStart, def._tableStart + def._tableLength),
		name: def.name
	}
}
