
import { PoolConnection } from 'mysql2/promise';
import { Logger } from './Logger';
import { QueryParameters } from './Queryable';
import { QueryExecutor } from './QueryExecutor';
import { TransactionError } from './TransactionError';

export class Transaction extends QueryExecutor {

	/** INTERNAL, DO NOT USE DIRECTLY */
	public static async start(poolConnection: PoolConnection, logger: Logger, mode?: 'READ' | 'WRITE') {
		const tx = new Transaction(poolConnection, logger);
		await tx.internalQuery(
			`START TRANSACTION${mode === 'READ' ? ' READ ONLY' : mode === 'WRITE' ? ' READ WRITE' : ''}`
		);
		return tx;
	}

	/** INTERNAL, DO NOT USE DIRECTLY */
	public static async commit(tx: Transaction) {
		if (!tx.inTx) {
			tx.closed = true;
			throw new TransactionError(`Transaction has been prematurely committed or rolled back`);
		}
		try {
			await tx.internalQuery(`COMMIT`);
		} finally {
			tx.closed = true;
		}
	}

	/** INTERNAL, DO NOT USE DIRECTLY */
	public static async rollback(tx: Transaction) {
		if (!tx.inTx) {
			tx.closed = true;
			throw new TransactionError(`Transaction has been prematurely committed or rolled back`);
		}
		try {
			await tx.internalQuery(`ROLLBACK`);
		} finally {
			tx.closed = true;
		}
	}

	public query(sql: string, parameters?: QueryParameters) {
		if (!this.inTx) {
			throw new TransactionError(`Transaction is inactive and can not perform operations`);
		}
		return super.query(sql, parameters);
	}

	public get isActive() {
		return this.inTx && !this.closed;
	}

	private internalQuery(sql: string) {
		return super.query(sql);
	}

}
